#-----------------------------------------------------------------------------
require 'sketchup.rb'
#-----------------------------------------------------------------------------

Sketchup.send_action(CMD_RUBY_CONSOLE)

module JBB_LayersPanel
	
	@model = Sketchup.active_model
	@layers = @model.layers
	@layerDictID = nil
	@dialog = nil
	
	class << self
		attr_accessor :model, :layers, :layerDictID, :dialog
	end
	
	
	### LAYER SERIALIZE ### ------------------------------------------------------
	
	def self.incLayerDictID
		@layerDictID = @layerDictID + 1
		@model.set_attribute("jbb_layerspanel", "layerDictID", @layerDictID) #Store incremented layerDictID in model attribute dict
	end#def
	
	def self.initializeLayerDictID
		if @layerDictID == nil
			if @model.get_attribute("jbb_layerspanel", "layerDictID") != nil #Get layerDictID from model if exists
				@layerDictID = @model.get_attribute("jbb_layerspanel", "layerDictID")
			else #Else, create it
				@layers[0].set_attribute("jbb_layerspanel", "ID", 0) #Give Layer0 ID 0
				@layerDictID = 0
			end#if
			self.incLayerDictID
			# puts "Current layerDictID : " + @layerDictID.to_s
		end#if
	end#def
	
	def self.IdLayer(layer) #Give a unique custom id to a layer
		if layer.get_attribute("jbb_layerspanel", "ID") != nil 
			#puts layer.name + " already IDed " + layer.get_attribute("jbb_layerspanel", "ID").to_s
		else
			layer.set_attribute("jbb_layerspanel", "ID", @layerDictID)
			#puts "layerDictID " + @layerDictID.to_s
			self.incLayerDictID
		end#if
	end#def
	
	
	### LAYER DELETE METHOD ### ------------------------------------------------------
	#Simple modification of TIG's snippet delete-layer.rb
	#Basically, move or delete layer content, then creates an entry for every layer except one to delete, then purge layers, then delete entries
	
	def self.deleteLayer(layer, delete_geometry=false)
		ents=@model.entities; defs=@model.definitions
		@model.start_operation("Delete layer")
		if delete_geometry
			allents=[]
			@model.entities.each{|e|allents<<e if e.valid? and e.respond_to?(:layer)and e.layer==layer}
			@model.definitions.each{|d|d.entities.each{|e|allents<<e if e.valid? and e.respond_to?(:layer)and e.layer==layer}}
			allents.each{|e|e.erase! if e.valid?}
		else ### move geom to Layer0 etc
			@model.entities.each{|e|e.layer=nil if e.respond_to?(:layer)and e.layer==layer}
			@model.definitions.each{|d|d.entities.each{|e|e.layer=nil if e.respond_to?(:layer)and e.layer==layer}}
		end#if
		group=@model.entities.add_group();gents=group.entities ### temporarily use other layers
		temp=gents.add_group()
		temp.layer=nil
		(@layers.to_a-[layer]).each{|layer|tc=temp.copy;tc.layer=layer}
		@model.active_layer=nil if @model.active_layer==layer ### ensure layer is not current layer
		@layers.purge_unused ### purge layer from browser
		group.erase! ### erase! the temporary layer user, use set as was.
		@model.commit_operation 
	end#def
  
  
  
	### WEBDIALOG & CALLBACKS ### ------------------------------------------------------
	
	# Create the WebDialog instance
	@dialog = UI::WebDialog.new("Layers Panel", false, "LayersPanel", 300, 300, 300, 200, true)
	@dialog.min_width = 200
	html_path = File.dirname( __FILE__ ) + "/layers Panel.html"
	@dialog.set_file(html_path)

	@dialog.add_action_callback("getModelLayers") do  |wdl, action|
		self.initializeLayerDictID
		
		serialized = @model.get_attribute("jbb_layerspanel", "serialized") #retreive string of serialized layers
		matches = serialized.to_s.scan(/(layer|group)\[(\d+)\]\=(\d+|null)/) #make an array of it
		
		matches.each do |match| 
			#match[0] : layer or group
			#match[1] : ID
			#match[2] : parent ID
			match = match.to_a
			
			if match[0].to_s == "layer" #if layer
				@layers.each{|layer| 
					if layer.get_attribute("jbb_layerspanel", "ID").to_i == match[1].to_i #if layer's dict ID == match ID
						#puts layer.name
						addLayerFromRuby = "addLayerFromRuby('#{layer.name}', '#{match[1]}', '#{match[2]}');"
						wdl.execute_script(addLayerFromRuby)
						break
					end#if
				}
			else #if group
				groupName = @model.get_attribute("jbb_layerspanel_groups", match[1])
				addGroup = "addGroup('#{groupName}', '#{match[1]}', '#{match[2]}');"
				wdl.execute_script(addGroup)
			end
		end#each
		
		@layers.each{|layer|
			self.IdLayer(layer)
			layerId = layer.get_attribute("jbb_layerspanel", "ID")
			addLayerFromRuby = "addLayerFromRuby('#{layer.name}', '#{layerId}');"
			wdl.execute_script(addLayerFromRuby)
			if layer.visible?
				showLayerFromRuby = "showLayerFromRuby('#{layerId}');"
				wdl.execute_script(showLayerFromRuby)
			else
				hideLayerFromRuby = "hideLayerFromRuby('#{layerId}');"
				wdl.execute_script(hideLayerFromRuby)
			end#if
		}
	end#callback getModelLayers

	@dialog.add_action_callback("getActiveLayer") do  |wdl, action|
		activeLayer = @model.active_layer.get_attribute("jbb_layerspanel", "ID")
		setActiveLayerFromRuby = "setActiveLayerFromRuby('#{activeLayer}');"
		wdl.execute_script(setActiveLayerFromRuby)
	end#callback getModelLayers

	@dialog.add_action_callback("setActiveLayerFromJS") do  |wdl, layerId|
		# puts layerId
		@layers.each{|layer| 
			if layer.get_attribute("jbb_layerspanel", "ID").to_i == layerId.to_i
				@model.active_layer = layer
				
				showLayerFromRuby = "showLayerFromRuby('#{layerId}');"
				@dialog.execute_script(showLayerFromRuby)
				break
			end#if
		}
	end#callback getModelLayers

	@dialog.add_action_callback("addLayerFromJS") do
		@layers.add @layers.unique_name
	end#callback addLayerFromJS

	@dialog.add_action_callback("renameLayerFromJS") do |wdl, layerNameS|
		@model.start_operation("Rename layer")
		hashStringLayerNames = eval( layerNameS.inspect.gsub(':','=>') ) #Convert Json string to Hash string
		hashLayerNames = eval(hashStringLayerNames) #Convert hash string to proper hash object
		layerId = hashLayerNames['layerID']
		newLayerName = hashLayerNames['newLayerName']
		@layers.each{|layer| 
			if layer.get_attribute("jbb_layerspanel", "ID").to_i == layerId.to_i
				layer.name = newLayerName
				break
			end#if
		}
		@model.commit_operation
	end#callback renameLayerFromJS

	@dialog.add_action_callback("checkLayerForContent") do |wdl, layerId|
		# puts layerId
		@layers.each{|layer| 
			if layer.get_attribute("jbb_layerspanel", "ID").to_i == layerId.to_i
				layerHasContent = false
				
				allents=[]
				@model.entities.each{|e|allents<<e if e.valid? and e.respond_to?(:layer)and e.layer==layer}
				@model.definitions.each{|d|d.entities.each{|e|allents<<e if e.valid? and e.respond_to?(:layer)and e.layer==layer}}
				if allents != [] #Layer has content
					layerHasContent = true
				end#if
				
				checkLayerForContent = "checkLayerForContent('#{layerId}', '#{layerHasContent}');"
				@dialog.execute_script(checkLayerForContent)
				break
			end#if
		}
	end#callback deleteLayerFromJS

	@dialog.add_action_callback("deleteLayerFromJS") do |wdl, layerId|
		@layers.each{|layer| 
			if layer.get_attribute("jbb_layerspanel", "ID").to_i == layerId.to_i
				self.deleteLayer(layer)
				break
			end#if
		}
	end#callback deleteLayerFromJS

	@dialog.add_action_callback("deleteLayer&GeomFromJS") do |wdl, layerId|
		@layers.each{|layer| 
			if layer.get_attribute("jbb_layerspanel", "ID").to_i == layerId.to_i
				self.deleteLayer(layer, true)
				break
			end#if
		}
	end#callback deleteLayerFromJS

	@dialog.add_action_callback("hideLayerFromJS") do |wdl, layerId|
		# puts layerId
		@layers.each{|layer| 
			if layer.get_attribute("jbb_layerspanel", "ID").to_i == layerId.to_i
				layer.visible = false
				break
			end#if
		}
	end#callback hideLayerFromJS

	@dialog.add_action_callback("showLayerFromJS") do |wdl, layerId|
		# puts layerId
		@layers.each{|layer| 
			if layer.get_attribute("jbb_layerspanel", "ID").to_i == layerId.to_i
				layer.visible = true
				break
			end#if
		}
	end#callback showLayerFromJS

	@dialog.add_action_callback("purgeLayersFromJS") do |wdl, act|
		@layers.purge_unused
	end#callback purgeLayersFromJS

	@dialog.add_action_callback("getLayerDictID") do |wdl, act|
		sendLayerDictID = "receiveLayerDictID('#{@layerDictID}');"
		@dialog.execute_script(sendLayerDictID)
		self.incLayerDictID
	end#callback getLayerDictID

	@dialog.add_action_callback("addGroup") do |wdl, groupName|
		@model.start_operation("Add group layer")
		@model.set_attribute("jbb_layerspanel_groups", @layerDictID, groupName) #Store group's name with ID
		@model.commit_operation
	end#callback addGroup

	@dialog.add_action_callback("renameGroup") do |wdl, renameGroup|
		@model.start_operation("Rename group layer")
		hashStringGroup = eval( renameGroup.inspect.gsub(':','=>') ) #Convert Json string to Hash string
		hashGroup = eval(hashStringGroup) #Convert hash string to proper hash object
		groupID = hashGroup['groupID']
		newGroupName = hashGroup['newGroupName']
		@model.set_attribute("jbb_layerspanel_groups", groupID, newGroupName) #Store new group's name from ID
		@model.commit_operation
	end#callback renameLayerFromJS

	@dialog.add_action_callback("storeSerialize") do |wdl, serialized|
		@model.set_attribute("jbb_layerspanel", "serialized", serialized) #Store serialized in model attribute dict
		# puts serialized
	end#callback storeSerialize

	@dialog.set_on_close do
		@dialog.execute_script("storeSerialize();") 
	end
	
  
  
	### @layersOBSERVER ### ------------------------------------------------------
	
	# Layers observer
	class JBB_LP_layersObserver < Sketchup::LayersObserver
	
		def onLayerAdded(layers, layer)
			JBB_LayersPanel.initializeLayerDictID
			JBB_LayersPanel.IdLayer(layer)
			layerIdForJS = layer.get_attribute("jbb_layerspanel", "ID")
			addLayerFromRuby = "addLayerFromRuby('#{layer.name}', '#{layerIdForJS}');"
			JBB_LayersPanel.dialog.execute_script(addLayerFromRuby)
			
			# Attach the entity observer.
			layer.add_observer(JBB_LP_EntityObserver.new)
		end#onLayerAdded
		
		def onLayerRemoved(layers, layer)
			layerId = layer.get_attribute("jbb_layerspanel", "ID")
			deleteLayerFromRuby = "deleteLayerFromRuby('#{layerId}');"
			JBB_LayersPanel.dialog.execute_script(deleteLayerFromRuby)
		end#onLayerRemoved
		
		
		def onCurrentLayerChanged(layers, layer)
			JBB_LayersPanel.dialog.execute_script('getActiveLayer();')
		end#onLayerRemoved
		
	end#JBB_LP_layersObserver
	
	# Attach the observer.
	@layers.add_observer(JBB_LP_layersObserver.new)
	
  
  
	### ENTITYOBSERVER ### ------------------------------------------------------
	#Watches for layers to be hidden or renamed (Which layers observer doesn't support)
	
	class JBB_LP_EntityObserver < Sketchup::EntityObserver
		def onChangeEntity(layer)
			# puts layer.name
			layerId = layer.get_attribute("jbb_layerspanel", "ID")
			
			if layer.visible?
				showLayerFromRuby = "showLayerFromRuby('#{layerId}');"
				JBB_LayersPanel.dialog.execute_script(showLayerFromRuby)
			else
				hideLayerFromRuby = "hideLayerFromRuby('#{layerId}');"
				JBB_LayersPanel.dialog.execute_script(hideLayerFromRuby)
			end#if
			
			renameLayerFromRuby = "renameLayerFromRuby('#{layerId}', '#{layer.name}');"
			JBB_LayersPanel.dialog.execute_script(renameLayerFromRuby)
		end
	end
	
	# Attach the observer to layer0
	@layers[0].add_observer(JBB_LP_EntityObserver.new)
	
  
  
	### APPOBSERVER ### ------------------------------------------------------
	
	class JBB_LP_AppObserver < Sketchup::AppObserver

		def onNewModel(newModel)
			JBB_LayersPanel.model = newModel
			JBB_LayersPanel.layers = newModel.layers
			
			JBB_LayersPanel.layerDictID = nil
			
			JBB_LayersPanel.model.add_observer(JBB_LP_ModelObserver.new)
			JBB_LayersPanel.model.pages.add_observer(JBB_LP_PagesObserver.new)
			JBB_LayersPanel.layers.add_observer(JBB_LP_layersObserver.new)
			
			JBB_LayersPanel.layers.each{|layer| 
				layer.add_observer(JBB_LP_EntityObserver.new)
			}
			
			JBB_LayersPanel.dialog.execute_script("reloadDialog();")
		end

		def onOpenModel(newModel)
			JBB_LayersPanel.model = newModel
			JBB_LayersPanel.layers = newModel.layers
			
			JBB_LayersPanel.layerDictID = nil
			
			JBB_LayersPanel.model.add_observer(JBB_LP_ModelObserver.new)
			JBB_LayersPanel.model.pages.add_observer(JBB_LP_PagesObserver.new)
			JBB_LayersPanel.layers.add_observer(JBB_LP_layersObserver.new)
			
			JBB_LayersPanel.layers.each{|layer| 
				layer.add_observer(JBB_LP_EntityObserver.new)
			}
			
			JBB_LayersPanel.dialog.execute_script("reloadDialog();")
		end

	end

	# Attach the observer
	Sketchup.add_observer(JBB_LP_AppObserver.new)
	
  
  
	### MODELOBSERVER ### ------------------------------------------------------
	
	class JBB_LP_ModelObserver < Sketchup::ModelObserver
		def onTransactionUndo(model)
			JBB_LayersPanel.dialog.execute_script("reloadDialog();")
		end#def
	end

	# Attach the observer
	@model.add_observer(JBB_LP_ModelObserver.new)
	
  
  
	### PAGESOBSERVER ### ------------------------------------------------------
	
	class JBB_LP_PagesObserver < Sketchup::PagesObserver
		def onContentsModified(pages)
			JBB_LayersPanel.dialog.execute_script("reloadDialog();")
		end#def
	end

	# Attach the observer
	@model.pages.add_observer(JBB_LP_PagesObserver.new)
  
  
  
	### MENU & TOOLBARS ### ------------------------------------------------------
	
	def self.toggle_layerspanel_dlg
		if @dialog.visible?
			@dialog.close
			false
		else
			@dialog.show()
			true
		end#if
	end#def
	
	def self.layerspanel_dlg_validation_proc
		if @dialog && @dialog.visible?
			MF_CHECKED
		else
			MF_UNCHECKED
		end#if
	end#def
  
	unless file_loaded?( __FILE__ )
		# Commands
		cmd = UI::Command.new( 'Layers Panel' ) { self.toggle_layerspanel_dlg }
		cmd.status_bar_text = 'Show or hide the Layers Panel.'
		cmd.small_icon = "lp_16.png"
		cmd.large_icon = "lp_24.png"
		cmd.tooltip = 'Layers Panel'
		cmd.set_validation_proc { self.layerspanel_dlg_validation_proc }
		cmd_toggle_layerspanel_dlg = cmd

		window_menu = UI.menu("Window")
		window_menu.add_item( cmd_toggle_layerspanel_dlg )
		
		layerspanel_tb = UI::Toolbar.new "Layers Panel"
		layerspanel_tb.add_item cmd_toggle_layerspanel_dlg
		layerspanel_tb.show
	end 
	
end#module

#-----------------------------------------------------------------------------
file_loaded( File.basename(__FILE__) )
